Ext.namespace('Zarafa.plugins.owncloudattachment.data');

/**
 * @class Zarafa.plugins.owncloudattachment.data.TreeLoader
 * @extends Ext.tree.TreeLoader
 *
 * OwncloudAttachments tree loader. Extends Ext treeloader to use Zarafa
 * specific requests.
 */
Zarafa.plugins.owncloudattachment.data.TreeLoader = Ext.extend(Ext.tree.TreeLoader, {

	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		Ext.applyIf(config, {
			directFn : this.loadNode.createDelegate(this)
		});

		Zarafa.plugins.owncloudattachment.data.TreeLoader.superclass.constructor.call(this, config);
	},

	/**
	 * Will do single request to owncloud module with provided nodeId and
	 * in case of success will load the content of this node.
	 *
	 * @param {Number} nodeId The id of node which content need to be loaded
	 * @param {Function} callback The function which need to be called after response received
	 */
	loadNode : function(nodeId, callback)
	{
		var self = this;
		var responseHandler = new Zarafa.plugins.owncloudattachment.data.ResponseHandler({
			successCallback: callback,
			nodeId: nodeId
		});
		
		container.getRequest().singleRequest(
			'owncloudmodule',
			'loadnode',
			{
				id : nodeId,
				uid: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/username'),
				pass: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/password'),
				oc_server: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/owncloud_server'),
				oc_base: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/owncloud_path'),
				oc_ssl: container.getSettingsModel().get('zarafa/v1/plugins/owncloudattachment/use_ssl')
			},
			responseHandler
		);
	}
});