<?php
/**
 * class.owncloudmodule.php, owncloud zarafa integration module
 *
 * Author: Christoph Haas <christoph.h@sprinternet.at>
 * Copyright (C) 2012 Christoph Haas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
require_once('Owncloud/webdav.php');
require_once('Owncloud/authexception.php');

/**
 * This module integrates Owncloud into attachment part of emails
 * @class OwncloudModule
 * @extends Module
 */
class OwncloudModule extends Module {

	private $wdc;				// webdav client
	private $initdone = false;
	private $DEBUG = false; 	// enable error_log debugging

	/**
	 * @constructor
	 * @param $id
	 * @param $data
	 */
	public function __construct($id, $data)
	{
			parent::Module($id, $data);	
	}

	/**
	 * Initialize PHP API of Owncloud
	 * @param $data
	 */
	private function bootstrap($data) {
		$this->wdc = new webdav_client();
		$this->wdc->set_server($data["oc_server"]);
		if($data["oc_ssl"] == true) {
			$this->wdc->set_port(443);
		} else {
			$this->wdc->set_port(80);
		}
		$this->wdc->set_ssl($data["oc_ssl"]);
		$this->wdc->set_pathprefix($data["oc_base"]);
		// use HTTP/1.1
		$this->wdc->set_protocol(1);
		// enable debugging
		$this->wdc->set_debug($this->DEBUG);
	}



	/**
	 * Executes all the actions in the $data variable.
	 * Exception part is used for authentication errors also
	 * @return boolean true on success or false on failure.
	 */
	public function execute()
	{
		$result = false;

		foreach($this->data as $actionType => $actionData)
		{			
			/* we have to fix this.... */
			//TODO: rewrite this dirty code...
			if(!$this->initdone && isset($actionData)) {
				$this->bootstrap($actionData);
				$this->initdone = true;
			}
			if(isset($actionType)) {
				try {
					if($this->DEBUG) {
						error_log("exec: " . $actionType);
					}
					switch($actionType)
					{
						case "loadnode":
							$result = $this->loadNode($actionData);
							break;
						case "download-to-tmp":
							$result = $this->downloadSelectedOwncloudFilesToTmp($actionData);
							break;
						default:
							$this->handleUnknownActionType($actionType);
					}

				} catch (MAPIException $e) {
					if($this->DEBUG) {
						error_log("mapi exception: " . $e->getMessage());
					}
				} catch (AuthException $e) {
					//TODO: rewrite this dirty code...
					if ($e->getCode() == 401) {
						$nodes["/"] = array(
							'id'	=>	"/",
							'path'	=>	"/",
							'text'	=>	$e->getCode() . " Username or Password wrong!",
							'filename'=> "error",
							'leaf'	=>	false,
							'checked'=>	false
						);						
					} else if ($e->getCode() == 403) {
						$nodes["/"] = array(
							'id'	=>	"/",
							'path'	=>	"/",
							'text'	=>	$e->getCode() . " Could not connect to Server!",
							'filename'=> "error",
							'leaf'	=>	false,
							'checked'=>	false
						);
					}
					
					$response['items'] = array_values($nodes);
					$response['status']	=	true;
					$this->addActionData("loadnode", $response);
					$GLOBALS["bus"]->addData($this->getResponseData());
					
					if($this->DEBUG) {
						error_log("auth exception: " . $e->getMessage());
					}
				}
			}
		}
		return $result;
	}

	/**
	 * loads content of current folder - list of folders and files from Owncloud
	 * @param {Array} $actionData
	 */
	public function loadNode($actionData)
	{
		if (!$this->wdc->open()) {
			if($this->DEBUG) {
				error_log('Error: could not open server connection');
			}
			throw new AuthException("403 Failed to connect",403);
		}
		
		/* logon to owncloud */
		$this->authorise($actionData);
		
		$nodeId = $actionData['id'];
		if($nodeId == 'root') {
			$basedir = "/";
			$dir = $this->wdc->ls($basedir);
		} else {
			$basedir = substr($nodeId,strlen($actionData["oc_base"]));
			$dir = $this->wdc->ls($basedir);
		}		
		$this->wdc->close();
		
		if($this->DEBUG) {
			error_log("parsing dir: " . $dir);
		}
		if($dir == -401) {	// unauthorised!
			throw new AuthException("401 unauthorised",401);
		} else {
			$response["items"] = $this->getFolderContent($basedir, $dir, $actionData);
		}
		if($this->DEBUG) {
			error_log("parsing done!");
		}
		
		$response['status']	=	true;
		$this->addActionData("loadnode", $response);
		$GLOBALS["bus"]->addData($this->getResponseData());
		
		if($this->DEBUG) {
			error_log("nodes loaded, bus data written!");
		}
	}

	/**
	 * form the structure needed for frontend
	 * for the list of folders and files
	 * @param basedir
	 * @param dir
	 * @param data
	 * @return {Array} nodes for current path folder
	 * @private
	 */
	private function getFolderContent($basedir, $dir, $data) {
		$nodes = array();
		
		foreach($dir as $node) {
			if($this->DEBUG) {
				error_log("parsing: " . $node['href'] . " in base: " . $basedir);
			}
			
			$filename =  basename($node['href']);
			if (strcmp(substr($node['href'],strlen($data["oc_base"])),$basedir) == 0) { // current dir...
				continue;
			}
			if (strcmp($node['resourcetype'],"collection") == 0) { // we have a folder
				$nodes[$node['href']] = array(
					'id'	=>	$node['href'],
					'path'	=>	dirname($node['href']),
					'text'	=>	$filename,
					'filename'=>$filename
				);
			} else {
				$nodes[$node['href']] = array(
					'id'	=>	$node['href'],
					'path'	=>	dirname($node['href']),
					'text'	=>	$filename . '(' . $node['getcontentlength'] . ')',
					'filename'=>$filename,
					'leaf'	=>	true,
					'checked'=>	false
				);
			}
		}
		return array_values($nodes);
	}

	/**
	 * Downloads file from the Owncloud service and saves it in tmp
	 * folder with unique name
	 * @param {Array} $actionData
	 * @private
	 */
	private function downloadSelectedOwncloudFilesToTmp($actionData) {
		$ids = $actionData['ids'];
		$dialogAttachmentId = $actionData['dialog_attachments'];
		$response = array();
		
		if (!$this->wdc->open()) {
			if($this->DEBUG) {
				error_log('Error: could not open server connection');
			}
			throw new AuthException("403 Failed to connect",403);
		}
		
		/* logon to owncloud */
		$this->authorise($actionData);
		
		$attachment_state = new AttachmentState();
		foreach ($ids as $id) {
			$file = substr($id,strlen($actionData["oc_base"]));
			$filename = basename($file);
			$tmpname = tempnam(TMP_PATH, stripslashes($filename));
			
			$path = dirname($file);
			
			if($this->DEBUG) {
				error_log("Downloading: " . $filename . " to: " . $tmpname);
			}
			/* download file from webdav */
			$buffer = null;
			$http_status = $this->wdc->get($file, $buffer);
			if($this->DEBUG) {
				error_log("http-status: " . $http_status);
				error_log("buffer: " . $buffer);
			}			
			
			$fhandle = fopen($tmpname,'w');
			fwrite($fhandle,$buffer,strlen($buffer));
			fclose($fhandle); 
			
			$response['items'][] = array(
				'name' => $filename,
				'size' => filesize($tmpname),
				'tmpname'=>$this->getFilenameFromPath($tmpname)
			);			
			$finfo = finfo_open(FILEINFO_MIME_TYPE);
			$attachment_state->addAttachmentFile($dialogAttachmentId, $this->getFilenameFromPath($tmpname), Array(
				"name"       => $filename,
				"size"       => filesize($tmpname),
				"type"       => finfo_file($finfo, $tmpname),
				"sourcetype" => 'default'
			));
			if($this->DEBUG) {
				error_log("Mime: " . finfo_file($finfo, $tmpname) . " filesize: " . filesize($tmpname));
			}
		}
		
		$this->wdc->close();
		$attachment_state->close();
		$response['status'] = true;
		$this->addActionData("downloadtotmp", $response);
		$GLOBALS["bus"]->addData($this->getResponseData());
	}

	/**
	 * @param {String} $path
	 * @return {String} filename
	 * @private
	 */
	private function getFilenameFromPath($path) {
		$pathParts = explode('/', $path);
		return end($pathParts);
	}

	/**
	 * Sets access token for Owncloud's WebDav Interface
	 * @param {Array} $data
	 * @private
	 */
	private function authorise($data) {
		$this->wdc->set_user($data["uid"]);
		$this->wdc->set_pass($data["pass"]);
		
		if($this->DEBUG) {
			error_log("auth for " . $data["uid"]);
		}
	}
};

?>
